@@ -652,10 +652,6 @@ int si_cs_send(struct conn_stream *cs)
 	int ret;
 	int did_send = 0;
 
-	/* We're already waiting to be able to send, give up */
-	if (si->wait_event.events & SUB_RETRY_SEND)
-		return 0;
-
 	if (conn->flags & CO_FL_ERROR || cs->flags & (CS_FL_ERROR|CS_FL_ERR_PENDING)) {
 		/* We're probably there because the tasklet was woken up,
 		 * but process_stream() ran before, detected there were an
@@ -669,6 +665,10 @@ int si_cs_send(struct conn_stream *cs)
 		return 1;
 	}
 
+	/* We're already waiting to be able to send, give up */
+	if (si->wait_event.events & SUB_RETRY_SEND)
+		return 0;
+
 	/* we might have been called just after an asynchronous shutw */
 	if (conn->flags & CO_FL_SOCK_WR_SH || oc->flags & CF_SHUTW)
 		return 1;
@@ -922,12 +922,6 @@ void si_sync_send(struct stream_interface *si)
 	if (!cs)
 		return;
 
-	if (cs->flags & (CS_FL_ERROR|CS_FL_ERR_PENDING))
-		return;
-
-	if (cs->conn->flags & CO_FL_ERROR)
-		return;
-
 	si_cs_send(cs);
 }
 
